@@ -548,8 +548,20 @@ update_mount (Mount *     mnt,
 		} else if (! strncmp (mnt->device, "LABEL=", 6)) {
 			dequote (mnt->device + 6);
 		} else {
+			char *colon;
 			dequote (mnt->device);
-			strip_slashes (mnt->device);
+			/* If our device name is in host:/path format, as is
+			 * commonly used for network filesystems, don't strip
+			 * trailing slashes if this is the entire path.  We
+			 * look for the colon starting from the end, so that
+			 * we correctly handle IPv6 addresses for the host
+			 * part.
+			 */
+			if ((colon = strrchr (mnt->device,':')) != NULL
+			    && colon[1] == '/')
+				strip_slashes (colon + 2);
+			else
+				strip_slashes (mnt->device);
 		}
 	}
 
@@ -672,8 +684,7 @@ cut_options (const void *parent,
 
 		va_copy (options, args);
 		while ((option = va_arg (options, const char *)) != NULL) {
-			if (j && strlen(option) == j
-				&& ! strncmp (opts + i, option, j))
+			if (j && ! strncmp (opts + i, option, j))
 				break;
 		}
 		va_end (options);
@@ -909,7 +920,7 @@ parse_mountinfo_file (int reparsed)
 		 * If invoked first time, just set the flag and leave complete
 		 * state update (including event trigger) to mark_mounted ().
 		 */
-		if (reparsed && ! mnt->pending_call) {
+		if (reparsed) {
 			mounted (mnt);
 		} else {
 			mnt->mounted = TRUE;
@@ -1321,11 +1332,12 @@ tag_mount (Mount *mnt,
 			nih_debug ("%s is root filesystem", MOUNT_NAME (mnt));
 			tag = TAG_LOCAL;
 		} else if (is_remote (mnt)) {
-			if ((! strcmp (mnt->mountpoint, "/usr"))
-			    || (! strcmp (mnt->mountpoint, "/var"))
-			    || (! strncmp (mnt->mountpoint, "/usr/", 5))
-			    || (! strncmp (mnt->mountpoint, "/var/", 5))
-			    || (has_option (mnt, "bootwait", FALSE)))
+			if (((! strcmp (mnt->mountpoint, "/usr"))
+			     || (! strcmp (mnt->mountpoint, "/var"))
+			     || (! strncmp (mnt->mountpoint, "/usr/", 5))
+			     || (! strncmp (mnt->mountpoint, "/var/", 5))
+			     || (has_option (mnt, "bootwait", FALSE)))
+			    && ! has_option (mnt, "nobootwait", FALSE))
 			{
 				tag = TAG_REMOTE;
 			} else {
@@ -1729,8 +1741,6 @@ try_mounts (void)
 			/* All mounts have been attempted, so wait for
 			 * pending events.
 			 */
-			int still_pending = 0;
-
 			NIH_LIST_FOREACH (mounts, iter) {
 				Mount           *mnt = (Mount *)iter;
 				DBusPendingCall *pending_call = mnt->pending_call;
@@ -1738,17 +1748,10 @@ try_mounts (void)
 				if (!pending_call)
 					continue;
 
-				if (! dbus_pending_call_get_completed (pending_call))
-				{
-					still_pending++;
-					continue;
-				}
 				dbus_pending_call_block (pending_call);
 				dbus_pending_call_unref (pending_call);
 				mnt->pending_call = NULL;
 			}
-			if (still_pending > 0)
-				return;
 
 			if (control_server) {
 				dbus_server_disconnect (control_server);
@@ -2303,19 +2306,9 @@ run_mount_finished (Mount *mnt,
 
 	/* Parse mountinfo to see what mount did; in particular to update
 	 * the type if multiple types are listed in fstab.
+	 * Let parse_mountinfo () invoke mounted () to update state.
 	 */
-	if (! mnt->mounted) {
-		parse_mountinfo ();
-		/* If the canonical path of the mount point doesn't match
-		 * what's in the fstab, reparsing /proc/mounts won't change
-		 * this mount but instead create a new record.  So handle
-		 * this case directly here.
-		 */
-		if (! mnt->mounted && ! mnt->pending_call)
-			mounted (mnt);
-	} else {
-		mounted (mnt);
-	}
+	parse_mountinfo ();
 }
 
 
@@ -2687,17 +2680,8 @@ emit_event (const char *name,
 
 	nih_assert (name != NULL);
 
-	if (mnt && cb) {
-		reply_data = NIH_MUST (nih_alloc (NULL, sizeof (EventReplyData)));
-		reply_data->mnt = mnt;
-		reply_data->handler = cb;
-	}
-
-	if (no_events) {
-		if (cb)
-			cb (reply_data, NULL);
+	if (no_events)
 		return;
-	}
 
 	/* Flush the Plymouth connection to ensure all updates are sent,
 	 * since the event may kill plymouth.
@@ -2731,6 +2715,9 @@ emit_event (const char *name,
 		nih_discard (var);
 	}
 
+	if (mnt && cb)
+		reply_data = NIH_MUST (nih_alloc (NULL, sizeof (EventReplyData)));
+	
 	pending_call = NIH_SHOULD (upstart_emit_event (upstart,
 						       name, env, reply_data ? TRUE : FALSE,
 						       reply_data ? cb : NULL,
@@ -2741,6 +2728,9 @@ emit_event (const char *name,
 	
 		if (reply_data) {
 
+			reply_data->mnt = mnt;
+			reply_data->handler = cb;
+
 			/* If previous event is still pending, wait for it. */
 			if (mnt->pending_call) {
 				DBusPendingCall *pending = mnt->pending_call;
@@ -3380,7 +3370,7 @@ plymouth_progress (Mount *mnt,
 	 */
 	update = NIH_MUST (nih_sprintf (NULL, "fsck:%s:%d:%s",
 					MOUNT_NAME (mnt), progress,
-					_("Checking disk %1$d of %2$d (%3$d %% complete)")));
+					_("Checking disk %1$d of %2$d (%3$d%% complete)")));
 
 	ply_boot_client_update_daemon (ply_boot_client, update,
 				       plymouth_response,
@@ -3847,7 +3837,7 @@ change_mount_device (const char *devname,
 				/* Change only if the requested device is
 				 * really any different than whats stored
 				 */
-				if (!strcmp (mnt->device, devname)) {
+				if (strcmp (mnt->device, devname)) {
 					char * newdev;
 					ret = 0;
 					newdev  = nih_strdup (mounts, devname);
@@ -3993,6 +3983,12 @@ main (int   argc,
 		exit (EXIT_ERROR);
 	}
 
+	/* SIGUSR1 tells us that a network device came up.  Install
+           the handler before daemonising so that the mountall-net job
+           won't kill us by sending USR1. */
+	nih_signal_set_handler (SIGUSR1, nih_signal_handler);
+	NIH_MUST (nih_signal_add_handler (NULL, SIGUSR1, usr1_handler, NULL));
+
 	/* Become daemon */
 	if (daemonise) {
 		pid_t pid;
@@ -4046,10 +4042,6 @@ main (int   argc,
 	nih_signal_set_handler (SIGTERM, nih_signal_handler);
 	NIH_MUST (nih_signal_add_handler (NULL, SIGTERM, nih_main_term_signal, NULL));
 
-	/* SIGUSR1 tells us that a network device came up */
-	nih_signal_set_handler (SIGUSR1, nih_signal_handler);
-	NIH_MUST (nih_signal_add_handler (NULL, SIGUSR1, usr1_handler, NULL));
-
 	/* Check for force-fsck on the kernel command line */
 	cmdline = fopen ("/proc/cmdline", "r");
 	if (cmdline) {
